#ifndef MAGIC_OVERLAY_RENDERER_H
#define MAGIC_OVERLAY_RENDERER_H
/*
MagicOverlayRenderer class
*/
#include <SupportDefs.h>
#include <MediaDefs.h>
#include <Bitmap.h>
#include <GraphicsDefs.h>
#include "VideoRenderer.h"

#define BM_DECODE 0
#define BM_DRAW 1
#define INT32 int32
#define SIZEOF sizeof

#define PRIO_DRAWER 20

// blit using DrawBitmap instead ot overlay
// to test the convertion code (thanks pel on beshare)
#define TEST_MAGIC_OVERLAY

class MagicOverlayRenderer : public VideoRenderer
{
public:
		MagicOverlayRenderer();
	virtual	~MagicOverlayRenderer();
	status_t	InitCheck() {return B_OK;};
	const char	*RendererName() const {return "magic overlay";};
	const char	*ColorSpaceName() const;
	color_space	ColorSpace();
	void		*Bits();
	status_t	LockBits();
	status_t	UnlockBits();
	
	// render the frame in buff, the next buffer to write to is returned (can be the same)
	// should block until the buffer returned is ready to be fulled
	void		*Draw(void *buff);
	
	status_t	Install(MediaView *view, media_format *format);
	status_t	Use();
	status_t	Dispose();
	color_space	NextColorSpace(int *cookie); // cookie = 0 first
	BRect		Bounds();
	int32		BytesPerRow();
private:
	static int32	DrawerThreadRGB16(void *arg);
	static int32	DrawerThreadYCbCr422(void *arg);
	thread_id	drawer;
	sem_id		decode_sem, draw_sem;
	int			cs_index;
//	int			avail; // the bitmap available for decoding
//	int		to_draw; // the bitmap to draw
	BBitmap		*bitmap[2];
	long		rgb_sz;
	long		drawbm_sz;
	BRect		*bitmapBounds;
	MediaView	*view;
	
	color_space	cspace_used; // internal color space
	
	int64		droppedframes;

// lookup tables
// use one table per lookup instead of one big table
	int32		*lt_r_y;
	int32		*lt_g_y;
	int32		*lt_b_y;
	int32		*lt_r_cb;
	int32		*lt_g_cb;
	int32		*lt_b_cb;
	int32		*lt_r_cr; // same as lt_b_cb
	int32		*lt_g_cr;
	int32		*lt_b_cr;

};


#endif // MAGIC_OVERLAY_RENDERER_H
